function [tout, yout] = frk45(FunFcn, t0, tfinal, y0, tol,trace)
% frk45.m Integrate a system of ordinary differential equations using
%	4th and 5th order Runge-Kutta formulas.  Variable stepsize
% The Fehlberg coefficients:
ceta = [1/4  3/8  12/13  1  1/2]';
alpha = [ [    1      0      0     0      0    0]/4
          [    3      9      0     0      0    0]/32
          [ 1932  -7200   7296     0      0    0]/2197
          [ 8341 -32832  29440  -845      0    0]/4104
          [-6080  41040 -28352  9295  -5643    0]/20520 ]';
beta = [ [902880  0  3953664  3855735  -1371249  277020]/7618050
          [ -2090  0    22528    21970    -15048  -27360]/752400 ]';
pow = 1/5; stages=6;
if nargin < 6, trace = 0; end
if nargin < 5, tol = 1.e-6; end
% Initialization
t = t0;hmax = (tfinal - t)/5;hmin = (tfinal - t)/20000;
h = (tfinal - t)/100; y = y0(:);f = y*zeros(1,stages);
tout = t; yout = y.'; tau = tol * max(norm(y, 'inf'), 1);
if trace
   clc, t, h, y
end
% The main loop
while abs(t- tfinal)> 1e-6  & (h >= hmin)
  if t + h > tfinal, h = tfinal - t; end
  % Compute the slopes
  temp = feval(FunFcn,t,y);
  f(:,1) = temp(:);
  for j = 1:stages-1;
    temp = feval(FunFcn, t+ceta(j)*h, y+h*f*alpha(:,j));
    f(:,j+1) = temp(:);
  end
  % Estimate the error and the acceptable error
  delta = norm(h*f*beta(:,2),'inf');
  tau = tol*max(norm(y,'inf'),1.0);
  % Update the solution only if the error is acceptable
   if delta <= tau
      t = t + h;
      y = y + h*f*beta(:,1);
      tout = [tout; t];
      yout = [yout; y.'];
    end
    if trace
      home, t, h, y
    end
    % Update the step size
    if delta ~= 0.0
       h = min(hmax, 0.8*h*(tau/delta)^pow);
    end
end;
if (t < tfinal)
  disp('SINGULARITY LIKELY.')
  t
end
